// File           : Hantune A2L parameter preparation.
// Version        : version 3, 15 dec 2017
//                  Contains specification and examples for 'special comments' to be placed in 
//                  source code. Using these special comments combined with addresses from a map 
//                  file, an a2l file for HANtune can be generated.


// Every Special Comment starts with '/*A2lPrep' and ends with '*/'. Everything between these 
// tokens will be interpreted as a special comment. And therefore these special comments can be 
// placed at will in Java or C(++) code.
// Special comments consists of the following 3 types of comments: 
// 1 Project specification; 
// 0 or more Signal specifications 
// 0 or more Parameter specifications.


/*********************** Project specification ****************************************************
 @ProjectId "<name>"        // Required; Unique string holding project name and unique version number 
                            // (preferably including a build timestamp).
 @AddressProjectId <address> // Field is required only when flashing a target.
                            // Item holds location where ProjectId string is stored in target memory. 
                            // Can be a value holding a direct address, or a string specifying the 
                            // name of the variable holding the ProjectId. In this case the name 
                            // should be present in the address map file and address can be derived.
 @CpuType "<cputype>"       // String that identifies the CPU.
 @EcuType "<ecutype>"       // String that describes the control unit.
 @Description <string>      // Optional; Free string for description
***************************************************************************************************/

// Example Project parameters
/*A2lPrep
 @ProjectId         "HANcoder_Olimexinoasym_softwareID_v1_251116_144552"
 @AddressProjectId  XcpStationId
 @CpuType           "STM32F103"
 @EcuType           "Olimexino"
 @Description       "MOD PAR TestComment"
*/

// results in the following A2l snippet:
/begin MOD_PAR "MOD PAR TestComment"
    ADDR_EPK        0x0801e004
    CPU_TYPE        "STM32F103"
    ECU             "Olimexino"
    EPK             "HANcoder_Olimexinoasym_softwareID_v1_251116_144552"
    SYSTEM_CONSTANT "RAM-FlashOffset"  "-402602344"
/end MOD_PAR



/*********************** Signal specification ******************************************************
 @Signal <name>         // Required; <name> must be present in address file (.MAP)
 @PrimitiveType <ItemPrimitiveType> // Required; May only by one of the following: 
                        // BOOLEAN|U_BYTE|S_BYTE|U_WORD16|S_WORD16|U_DWORD32|S_DWORD32|U_QWORD64|
                        // S_QWORD64|FLOAT32_IEEE|FLOAT64_IEEE
 @StepSize <step>       // Optional, default=1.0; TargetValue = (DispVal - <offset>) / <step>;  
                        // DispVal = Targetvalue x <step> + <offset>
 @Offset <offset>       // Optional, default=0.0; <step> and <offset> are in physical units of 
                        // displayed value
 @DisplayFormat <display> // Optional, default=DEC; One of: DEC|FLOAT(1|2|3|4)|HEX|<formatString>;  
                        // example: "%2.3" or FLOAT2
 @UnitText "<unit>"     // Optional, default=""
 @LowerLimit <value>    // Optional, default=minimum PrimitiveType. Limit value must be specified in 
                        // Target units
 @UpperLimit <value>    // Optional, default=maximum PrimitiveType. Limit value must be specified in 
                        //Target units
 @Description "<string>"  // Optional, default=""; Example: "Free text"
 ***************************************************************************************************/

//Example sensor value
//sensor 0..250 (stored in byte, 251..255 not used) corresponds to -25..100 degr. centigrade.           
/*A2lPrep
 @Signal TempSensor
 @PrimitiveType U_BYTE
 @StepSize 0.5
 @Offset -25.0
 @DisplayFormat FLOAT1
 @UnitText "deg.C"
 @LowerLimit 0
 @UpperLimit 250
 @Description: "sensor 0..250 (stored in byte, 251..255 not used) corresponds to -25..100 degr. centigrade."
*/

// results in the following A2l snippet:
/begin MEASUREMENT
      /* Name                   */      TempSensor
      /* Long identifier        */      "sensor 0..250 (stored in byte, 251..255 not used) corresponds to -25..100 degr. centigrade."
      /* Data type              */      U_BYTE
      /* Conversion method      */      TempSensor_compu
      /* Resolution (Not used)  */      0
      /* Accuracy (Not used)    */      0
      /* Lower limit            */      -25
      /* Upper limit            */      100
      ECU_ADDRESS                       0x2000c57c
/end MEASUREMENT

/begin COMPU_METHOD
      /* Name of CompuMethod    */      TempSensor_compu
      /* Long identifier        */      "Q = V*0.5 -25.0"
      /* Conversion Type        */      RAT_FUNC
      /* Format                 */      "%2.1"
      /* Units                  */      "deg.C"
      /* Coefficients           */      COEFFS 0 0.5 -25.0 0 0 1.000000
/end COMPU_METHOD



/**************************** Parameter specification **********************************************
 @Parameter     <name>  // Required; <name> must be present in address file (.MAP)
 @Variety       <var>   // Required; Must be one of: VALUE|CURVE|AXIS_PTS|MAP
 @PrimitiveType <ItemPrimitiveType> // Required; May only by one of the following: 
                        // BOOLEAN|U_BYTE|S_BYTE|U_WORD16|S_WORD16|U_DWORD32|S_DWORD32|U_QWORD64|
                        // S_QWORD64|FLOAT32_IEEE|FLOAT64_IEEE
 @NumberOfAxisPts <value> // integer > 0; Required by AXIS_PTS only
 @StepSize <step>       // Optional default=1.0; TargetValue = (DispVal - <offset>) / <step>;  
                        // DispVal = Targetvalue x <step> + <offset>
 @Offset <offset>       // Optional default=0.0; <step> and <offset> are in physical units of 
                        // displayed value
 @DisplayFormat <display> // DEC|FLOAT(1|2|3|4)|HEX|<formatString>; Optional default=DEC; 
                        // example: "%2.3" or FLOAT2
 @UnitText "<unit>"     // Optional, default=""
 @LowerLimit <value>    // Optional, default=minimum PrimitiveType; holds value in Target units
 @UpperLimit <value>    // Optional, default=maximum PrimitiveType; holds value in Target units
 @Description "<string>"// Optional, default=""; Example: "Free text"
 @IndexTable <name>     // Required when variety is CURVE; <name> must hold name of AXIS_PTS table
 @IndexTableColsY <name> // Required when variety is MAP; <name> must hold name of Y AXIS_PTS table
 @IndexTableRowsX <name> // Required when variety is MAP; <name> must hold name of X AXIS_PTS table
 */

 
/////////// Example Parameter VALUE special comment:
// 100..4000000100 corresponds to 0..4000 volt
/*A2lPrep
 @Parameter  ParVoltage
 @Variety VALUE
 @PrimitiveType U_DWORD32
 @StepSize 1.0e-06
 @Offset -1.0e-4
 @DisplayFormat  "%9.12"
 @UnitText "V"
 @LowerLimit 100
 @UpperLimit 4000000100
 @Description: "Step: 1.0e-06; Min: 0; Max: 4000.000000 V"
*/

//Yields the following Characteristic and compu_method
/begin CHARACTERISTIC 
  /* Name                   */      ParVoltage
  /* Long Identifier        */      ""
  /* Type                   */      VALUE 
  /* ECU Address            */      0x200003b0 
  /* Record Layout          */      Scalar_U_DWORD32 
  /* Maximum Difference     */      0 
  /* Conversion Method      */      ParVoltage_CompuMethod
  /* Lower Limit            */      0 
  /* Upper Limit            */      4000.0
/end CHARACTERISTIC


// COEFF[1]:  A2lPrep.StepSize
// COEFF[2]:  A2lPrep.Offset
// COEFF[5]:  1.0000
/begin COMPU_METHOD
  /* Name of CompuMethod    */      DTC_Fixdt0sig32bit12fracPlus10_CompuMethod
  /* Long identifier        */      "Q = V*1.0e-6 - 1.0e-4"
  /* Conversion Type        */      RAT_FUNC
  /* Format                 */      "%9.12"
  /* Units                  */      "V"
  /* Coefficients           */      COEFFS 0 1.0e-6 -1.0e-4 0 0 1.000000
/end COMPU_METHOD

/begin RECORD_LAYOUT Scalar_U_DWORD32
  FNC_VALUES  1 ULONG COLUMN_DIR DIRECT
/end   RECORD_LAYOUT





///////////// Example parameter CURVE (must be accompanied by 1 AXIS_PTS table):
// VolumetricEffMultTable table target value 0..64000 corresponds to 0..1.600 mult
// CoolantTemp_Lookup target value 0..250 corresponds to -40.0..160.0 degr.C
/*A2lPrep
 @Parameter VolumetricEffMultTable 
 @Variety CURVE
 @PrimitiveType U_WORD16
 @StepSize 2.5e-5 // = 1/40000
 @Offset   0
 @DisplayFormat "%1.5"
 @UnitText "Mult"
 @LowerLimit 0
 @UpperLimit 64000
 @Description: "Step: 0.000025; Min: 0; Max: 1.6"
 @IndexTable CoolantTemp_Lookup

*/

/*A2lPrep
 @Parameter CoolantTemp_Lookup
 @Variety AXIS_PTS
 @PrimitiveType U_BYTE
 @NumberOfAxisPts 6
 @StepSize 0.8
 @Offset   -40
 @DisplayFormat FLOAT1
 @UnitText "Degr.C"
 @LowerLimit 0
 @UpperLimit 250
 @Description: "Step: 0.8; Min: -40.0; Max: 160.0"
*/

//Yields the following Characteristic and compu_methods
/begin CHARACTERISTIC 
  /* Name                   */      VolumetricEffMultTable
  /* Long Identifier        */      "Step: 0.000025; Min: 0; Max: 1.6"
  /* Type                   */      CURVE 
  /* ECU Address            */      0x200003b0 
  /* Record Layout          */      Scalar_U_WORD16
  /* Maximum Difference     */      0 
  /* Conversion Method      */      VolumetricEffMultTable_CompuMethod
  /* Lower Limit            */      0 
  /* Upper Limit            */      4000.0
  /begin AXIS_DESCR
    /* Axis Type            */      COM_AXIS
    /* Reference to Input   */      NO_INPUT_QUANTITY
    /* Conversion Method    */      CoolantTemp_Lookup_CompuMethod
    /* Number of Axis Pts   */      6
    /* Lower Limit          */      -40.0
    /* Upper Limit          */      160.0
    AXIS_PTS_REF                    CoolantTemp_Lookup
  /end AXIS_DESCR
/end CHARACTERISTIC

/begin AXIS_PTS
  /* Name                   */      CoolantTemp_Lookup
  /* Long Identifier        */      "Step: 0.8; Min: -40.0; Max: 160.0"
  /* ECU Address            */      0x20000128
  /* Input Quantity         */      NO_INPUT_QUANTITY
  /* Record Layout          */      Lookup_U_BYTE
  /* Maximum Difference     */      0
  /* Conversion Method      */      CoolantTemp_Lookup_CompuMethod
  /* Number of Axis Pts     */      6
  /* Lower Limit            */      -40.0
  /* Upper Limit            */      160.0
/end AXIS_PTS
    
/begin COMPU_METHOD
  /* Name of CompuMethod    */      VolumetricEffMultTable_CompuMethod
  /* Long identifier        */      "Q = V*2.5e-5"
  /* Conversion Type        */      RAT_FUNC
  /* Format                 */      "%1.5"
  /* Units                  */      "Mult"
  /* Coefficients           */      COEFFS 0 2.5e-5 0.0 0 0 1.000000
/end COMPU_METHOD

/begin COMPU_METHOD
  /* Name of CompuMethod    */      CoolantTemp_Lookup_CompuMethod
  /* Long identifier        */      "Q = V*0.8 - 40.0"
  /* Conversion Type        */      RAT_FUNC
  /* Format                 */      "%3.1"
  /* Units                  */      "Degr.C"
  /* Coefficients           */      COEFFS 0 0.8 -40.0 0 0 1.000000
/end COMPU_METHOD

/begin RECORD_LAYOUT Scalar_U_WORD16
  FNC_VALUES  1 UWORD COLUMN_DIR DIRECT
/end   RECORD_LAYOUT

/begin RECORD_LAYOUT Lookup_U_BYTE
  FNC_VALUES  1 UBYTE COLUMN_DIR DIRECT
/end   RECORD_LAYOUT



////////////// Example parameter MAP (must be accompagnied by 2 AXIS_PTS tables):
// GasolineInjTable table target value 0..64000 corresponds to 0..64.000 ms
// ThrottlePosSens_Lookup row target value 0..250 corresponds to 0..100.0 %
// ManifoldAirPress_Lookup column target value 0..64000 corresponds to 0..250 kPa
/*A2lPrep
 @Parameter GasolineInjTable
 @Variety MAP
 @PrimitiveType U_WORD16
 @StepSize 0.001
 @Offset 0.0
 @DisplayFormat FLOAT3
 @UnitText "ms"
 @LowerLimit 0
 @UpperLimit 64000
 @Description "Step: 0.001; Min: 0; Max: 64.000 ms"
 @IndexTableColsY ManifoldAirPress_Lookup
 @IndexTableRowsX ThrottlePosSens_Lookup
*/

/*A2lPrep
 @Parameter ThrottlePosSens_Lookup
 @Variety AXIS_PTS
 @PrimitiveType U_BYTE
 @NumberOfAxisPts 10
 @StepSize 0.4
 @Offset 0.0
 @DisplayFormat FLOAT1
 @UnitText "%"
 @LowerLimit 0
 @UpperLimit 250
 @Description "Step: 0.4; Min: 0.000; Max: 100.0 %" 
*/

/*A2lPrep
 @Parameter ManifoldAirPress_Lookup
 @Variety AXIS_PTS
 @PrimitiveType U_WORD16
 @NumberOfAxisPts 8
 @StepSize 0.00390625
 @Offset 0.0
 @DisplayFormat FLOAT3
 @UnitText "kPa"
 @LowerLimit 0
 @UpperLimit 64000
 @Description "Step: 0.00390625; Min: 0.000; Max: 250.000 kPa"
*/

//Yields the following Characteristic and compu_methods
/begin CHARACTERISTIC 
  /* Name                   */      GasolineInjTable
  /* Long Identifier        */      "Step: 0.001; Min: 0; Max: 64.000 ms"
  /* Type                   */      MAP
  /* ECU Address            */      0x200003b0 
  /* Record Layout          */      Scalar_U_WORD16
  /* Maximum Difference     */      0 
  /* Conversion Method      */      GasolineInjTable_CompuMethod
  /* Lower Limit            */      0 
  /* Upper Limit            */      64.000
  /begin AXIS_DESCR
    /* Description of Y-Axis Points */
    /* Axis Type            */      COM_AXIS
    /* Reference to Input   */      NO_INPUT_QUANTITY
    /* Conversion Method    */      ThrottlePosSens_Lookup_CompuMethod
    /* Number of Axis Pts   */      10
    /* Lower Limit          */      0
    /* Upper Limit          */      100.0
    AXIS_PTS_REF                    ThrottlePosSens_Lookup
  /end AXIS_DESCR
  /begin AXIS_DESCR
    /* Description of X-Axis Points */
    /* Axis Type            */      COM_AXIS
    /* Reference to Input   */      NO_INPUT_QUANTITY
    /* Conversion Method    */      ManifoldAirPress_Lookup_CompuMethod
    /* Number of Axis Pts   */      8
    /* Lower Limit          */      0
    /* Upper Limit          */      250.000
    AXIS_PTS_REF                    ManifoldAirPress_Lookup
  /end AXIS_DESCR
/end CHARACTERISTIC

/begin AXIS_PTS
  /* Name                   */      ThrottlePosSens_Lookup
  /* Long Identifier        */      "Step: 0.4; Min: 0.000; Max: 100.0 %" 
  /* ECU Address            */      0x20001128
  /* Input Quantity         */      NO_INPUT_QUANTITY
  /* Record Layout          */      Scalar_U_BYTE
  /* Maximum Difference     */      0
  /* Conversion Method      */      ThrottlePosSens_Lookup_CompuMethod
  /* Number of Axis Pts     */      6
  /* Lower Limit            */      0.000
  /* Upper Limit            */      100.0
/end AXIS_PTS
    
/begin AXIS_PTS
  /* Name                   */      ManifoldAirPress_Lookup
  /* Long Identifier        */      "Step: 0.00390625; Min: 0.000; Max: 250.000 kPa" 
  /* ECU Address            */      0x20001128
  /* Input Quantity         */      NO_INPUT_QUANTITY
  /* Record Layout          */      Scalar_U_WORD16
  /* Maximum Difference     */      0
  /* Conversion Method      */      ManifoldAirPress_Lookup_CompuMethod
  /* Number of Axis Pts     */      8
  /* Lower Limit            */      0.00
  /* Upper Limit            */      250.000
/end AXIS_PTS
    

/begin COMPU_METHOD
  /* Name of CompuMethod    */      GasolineInjTable_CompuMethod
  /* Long identifier        */      "Q = V*0.001"
  /* Conversion Type        */      RAT_FUNC
  /* Format                 */      "%3.3"
  /* Units                  */      "ms"
  /* Coefficients           */      COEFFS 0 0.001 0.0 0 0 1.000000
/end COMPU_METHOD

/begin COMPU_METHOD
  /* Name of CompuMethod    */      ThrottlePosSens_Lookup_CompuMethod
  /* Long identifier        */      "Q = V*0.001"
  /* Conversion Type        */      RAT_FUNC
  /* Format                 */      "%3.1"
  /* Units                  */      "%"
  /* Coefficients           */      COEFFS 0 0.4 0.0 0 0 1.000000
/end COMPU_METHOD

/begin COMPU_METHOD
  /* Name of CompuMethod    */      ManifoldAirPress_Lookup_CompuMethod
  /* Long identifier        */      "Q = V*0.00390625"
  /* Conversion Type        */      RAT_FUNC
  /* Format                 */      "%3.3"
  /* Units                  */      "kPa"
  /* Coefficients           */      COEFFS 0 0.00390625 0.0 0 0 1.000000
/end COMPU_METHOD

/begin RECORD_LAYOUT Scalar_U_WORD16
  FNC_VALUES  1 UWORD COLUMN_DIR DIRECT
/end   RECORD_LAYOUT

/begin RECORD_LAYOUT Scalar_U_BYTE
  FNC_VALUES  1 UBYTE COLUMN_DIR DIRECT
/end   RECORD_LAYOUT


// How optional parameters are handled for MOD_PAR section in A2l file.
For basic calibration in HANtune, the A2l Project name (EPROM ID) needs to be linked with the target 
currently connected. Therefore the @ProjectId is required in the Spectial comment section.
The content of @ProjectId will be placed in the A2l file in the ProjectName but ALSO in the 
MOD_PAR EPK. When both are present (and possibly not equal), HANtune uses the EPK string in the 
MOD_PAR section.

When calibration also needs to be flashed using HANtune, the MOD_PAR names ADDR_EPK and the SYSTEM_CONSTANT 
"RAM-FlashOffset" need to be present in te MOD_PAR section. ADDR_EPK holds the start-address of the 
EPROM ID in a Motorola srec or Intel hex file.
Therefore the optional parameter @AddressProjectId needs to be present in the special comment 
section where @ProjectId has been specified. When @AddressProjectId has been specified, A2lPrep 
tries to calculate the RAM-FlashOffset from the section parameters found in the .map file. When 
both have been found, the MOD_PAR section will be filled with ADDR_EPK and RAM-FlashOffset.
A2lPrep will issue a warning when the RAM-FlashOffset cannot be calculated. In that situation, the 
ADDR_EPK and RAM-FlashOffset will not be present in the MOD_PAR section. HANtune can only perform 
basic calibration operation. Flashing will not possible using this A2l file.

As a third step, for RC28 and RC36 targets only: 
When ADDR_EPK and RAM-FlashOffset have been succesfully found, A2lPrep will search the given .map 
file(s) for the occurence of several CRC32 related parameters. When these have been found, flashing 
of an RC28/RC36 target will be possible using an Intel hex file. Several CRC32 related parameters 
will be placed in the MOD_PAR section (CRC_END_ADDR, CRC_PATCH_ADDR, CRC_START_ADDR, PCODE_START 
and PCODE_END).

Summarizing:
Special comment   |          | ASAP2 file                                     | HANtune function |
--------------------------------------------------------------------------------------------------
@ProjectId <name> | required | PROJECT ProjectName and MOD_PAR EPK            | basic calibration|
@AddressProjectId | optional | MOD_PAR ADDR_EPK and SYSTEM_CONSTANT           | Flashing srec    |
                             | "RAM-FlashOffset" -> if RAM-FlashOffset cannot |                  |
                             | be found/calculated, A2lPrep will issue a      |                  |
                             | warning, A2l still generated without MOD_PAR   |                  |
 All CRC parameters found    | MOD_PAR section will be extended with 5 CRC par| Flashing         |
                                                                              | RC28/RC36 using  |
                                                                              | Intel ihex       |                      

